/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.mda.fileresolver;

import java.util.List;

import org.tinygroup.fileresolver.impl.AbstractFileProcessor;
import org.tinygroup.logger.LogLevel;
import org.tinygroup.mda.ModelManager;
import org.tinygroup.mda.ModelManagerFactory;
import org.tinygroup.mda.config.view.ViewModel;
import org.tinygroup.vfs.FileObject;
import org.tinygroup.xstream.XStreamFactory;

import com.thoughtworks.xstream.XStream;

public class ViewModelFileProcessor extends AbstractFileProcessor {
	
	private static final String VIEW_EXT_FILENAME = ".viewmodel.xml";

	public List<FileObject> getEntityFiles() {
		return fileObjects;
	}

	public boolean isMatch(FileObject fileObject) {
		return fileObject.getFileName().endsWith(VIEW_EXT_FILENAME);
	}

	public void process(ClassLoader loader) {
		ModelManager manager = ModelManagerFactory.getManager();
		XStream stream = XStreamFactory
				.getXStream(ModelManager.MDA_XSTREAM_PACKAGENAME);
		for (FileObject fileObject : fileObjects) {
			try {
				logger.logMessage(LogLevel.INFO, "正在读取视图模型文件[{0}]",
						fileObject.getAbsolutePath());
				ViewModel model = (ViewModel) stream.fromXML(fileObject
						.getInputStream());
				manager.addViewModel(model);
				logger.logMessage(LogLevel.INFO, "读取并添加视图模型文件[{0}]结束",
						fileObject.getAbsolutePath());
			} catch (Exception e) {
				logger.errorMessage("读取视图模型文件[{0}]出错", e, fileObject.getAbsolutePath());
			}
		}

	}


	public void process() {
		process(this.getClass().getClassLoader());
	}


}
