/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.mda.flow.impl;

import org.tinygroup.context.Context;
import org.tinygroup.context.impl.ContextImpl;
import org.tinygroup.logger.Logger;
import org.tinygroup.logger.LoggerFactory;
import org.tinygroup.mda.MDAFileCreator;
import org.tinygroup.mda.config.entity.EntityField;
import org.tinygroup.mda.config.entity.EntityGroup;
import org.tinygroup.mda.config.entity.EntityModel;
import org.tinygroup.mda.config.entity.Operation;
import org.tinygroup.mda.config.entity.View;
import org.tinygroup.mda.config.trans.MDACreatorConfig;
import org.tinygroup.mda.config.trans.MDACreators;
import org.tinygroup.mda.config.view.ViewModel;
import org.tinygroup.mda.javabean.BeanDetial;
import org.tinygroup.mda.javabean.JavaFileCreatorHelper;
import org.tinygroup.mda.javabean.impl.MDATemplateHelper;
import org.tinygroup.mda.util.FileUtil;
import org.tinygroup.mda.util.MDANameUtil;
import org.tinygroup.mda.util.ModelTransUtil;
import org.tinygroup.mda.util.OperationType;

public class EntityFlowFileVelocityCreatorImpl extends MDATemplateHelper implements
		MDAFileCreator<EntityModel> {

	private final String DEFAULT_ADD_TEMPLATE = "/flowtemplate/flowAdd.mdatemplate";
	private final String DEFAULT_UPDATE_TEMPLATE = "/flowtemplate/flowUpdate.mdatemplate";
	private final String DEFAULT_DELETE_TEMPLATE = "/flowtemplate/flowDelete.mdatemplate";
	private final String DEFAULT_QUERY_TEMPLATE = "/flowtemplate/flowQuery.mdatemplate";
	private final String DEFAULT_VIEW_TEMPLATE = "/flowtemplate/flowQuery.mdatemplate";
	private final String DEFAULT_SINGLE_QUERY_TEMPLATE = "/flowtemplate/flowDefaultQuery.mdatemplate";
	private final String DEFAULT_VIEW_QUERY_TEMPLATE = "/flowtemplate/flowViewQuery.mdatemplate";
	private final String ADD_TEMPLATE = "add_template";
	private final String UPDATE_TEMPLATE = "update_template";
	private final String DELETE_TEMPLATE = "delete_template";
	private final String QUERY_TEMPLATE = "query_template";
	private final String VIEW_TEMPLATE = "view_template";
	private final String VIEW_QUERY_TEMPLATE = "view_query_template";
	private final String SINGLE_QUERY_TEMPLATE = "single_query_template";
	// private final String PAGING_TEMPLATE = "/flowPagingQuery.mdatemplate";
	private static Logger logger = LoggerFactory
			.getLogger(EntityFlowFileVelocityCreatorImpl.class);

	public void createFile(EntityModel model, BeanDetial bean, FileUtil fileUtil) {
		Context context = getContext(model,bean);
		EntityField pField = null;
		for(EntityGroup group:model.getGroups()){
			for(EntityField field:group.getFields()){
				if(field.isPrimary())
					pField = field;
			}
		}
		if(pField == null){
			throw new RuntimeException("主键不存在");
		}
		context.put("primaryField", pField);
		createViewDefaultFlowFile(fileUtil, model, context);
		for (Operation op : model.getOperations()) {
			createFlowFile(fileUtil, model, op, context);
		}
		for(View view:model.getViews()){
			createViewFlowFile(fileUtil, model, view, context);
		}

	}
	private void createViewDefaultFlowFile(FileUtil fileUtil, EntityModel model,
			Context context) {
		//String flowId = MDANameUtil.getFlowId(model);
		String flowId = MDANameUtil.getFlowId(model.getName());
		context.put("flowId", flowId);
		// 获取模板路径
		String templatePath = templateMap.get(SINGLE_QUERY_TEMPLATE);
		try {
			String flowContent = ModelTransUtil
					.parseFile(context, templatePath);
			fileUtil.writePageFlowFile(MDANameUtil.getFlowPackageName(model),
					flowId, flowContent);
		} catch (Exception e) {
			logger.errorMessage("为模型视图生成默认查询Flow时出错,模型:{0}", e,
					model.getName());
		}
	}

	private void createViewFlowFile(FileUtil fileUtil, EntityModel model,
			View view, Context context) {
//		String flowId = MDANameUtil.getFlowId(model, view);
		String flowId = MDANameUtil.getFlowId(model.getName(), view.getName());
		context.put("flowId", flowId);
		context.put("pagePath", MDANameUtil.getPagePath(model, view.getName()));
		// 获取模板路径
		String templatePath = templateMap.get(VIEW_QUERY_TEMPLATE);
		try {
			String flowContent = ModelTransUtil
					.parseFile(context, templatePath);
			fileUtil.writePageFlowFile(MDANameUtil.getFlowPackageName(model),
					flowId, flowContent);
		} catch (Exception e) {
			logger.errorMessage("为模型视图生成Flow时出错,模型:{0},视图:{1}", e,
					model.getName(), view.getName());
		}
	}
	
	private void createFlowFile(FileUtil fileUtil, EntityModel model,
			Operation op, Context context) {
//		String flowId = MDANameUtil.getFlowId(model, op);
		String flowId = MDANameUtil.getFlowId(model.getName(), op.getName());
		context.put("flowId", flowId);
		context.put("pagePath", MDANameUtil.getPagePath(model, op.getName()));
		// 获取模板路径
		String templatePath = getFlowTemplate(op.getType());
		try {
			String flowContent = ModelTransUtil
					.parseFile(context, templatePath);
			fileUtil.writePageFlowFile(MDANameUtil.getFlowPackageName(model),
					flowId, flowContent);
		} catch (Exception e) {
			logger.errorMessage("为模型操作生成Flow时出错,模型:{0},操作:{1}", e,
					model.getName(), op.getName());
		}
	}

		
	


	/**
	 * 设置上下文
	 * 
	 * @param bean
	 * @return
	 */
	private Context getContext(EntityModel model,BeanDetial bean) {
		Context context = new ContextImpl();
		context.put("model", model);
		context.put("beanFullName", MDANameUtil.getBeanFullName(bean));
		context.put("bean", bean);
		context.put("beanVarName", MDANameUtil.getMDAName(bean.getName()));
		context.put(JavaFileCreatorHelper.SERVICE_INTERFACE_PACKAGE_NAME,
				MDANameUtil.getServiceInterfacePackage(bean.getPackageName()));
		context.put(JavaFileCreatorHelper.SERVICE_INTERFACE_NAME, MDANameUtil.getServiceInterfaceName(bean.getName()));

		return context;
	}

	/**
	 * 获取不同操作对应的模板路径
	 * 
	 * @param opType
	 * @return
	 */
	private String getFlowTemplate(String opType) {
		if (opType.equals(OperationType.ADD.toString())) {
			return templateMap.get(ADD_TEMPLATE);
		} else if (opType.equals(OperationType.DELETE.toString())) {
			return templateMap.get(DELETE_TEMPLATE);
		} else if (opType.equals(OperationType.QUERY.toString())) {
			return templateMap.get(QUERY_TEMPLATE);
		} else if (opType.equals(OperationType.UPDATE.toString())) {
			return templateMap.get(UPDATE_TEMPLATE);
		} else if (opType.equals(OperationType.VIEW.toString())) {
			return templateMap.get(VIEW_TEMPLATE);
		}
		return "";
	}

	public void createFile(ViewModel model, BeanDetial bean, FileUtil fileUtil) {

	}

	public void setCreatorConfig(MDACreatorConfig creator,MDACreators creators) {
		setTemplateMap(ModelTransUtil.getTemplateMap(creator));
	}

	protected void initTemplate() {
		checkTemplate(VIEW_TEMPLATE, DEFAULT_VIEW_TEMPLATE);
		checkTemplate(ADD_TEMPLATE, DEFAULT_ADD_TEMPLATE);
		checkTemplate(DELETE_TEMPLATE, DEFAULT_DELETE_TEMPLATE);
		checkTemplate(QUERY_TEMPLATE, DEFAULT_QUERY_TEMPLATE);
		checkTemplate(VIEW_QUERY_TEMPLATE, DEFAULT_VIEW_QUERY_TEMPLATE);
		checkTemplate(UPDATE_TEMPLATE, DEFAULT_UPDATE_TEMPLATE);
		checkTemplate(SINGLE_QUERY_TEMPLATE, DEFAULT_SINGLE_QUERY_TEMPLATE);
	}

}
