/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.mda.page.impl;

import org.tinygroup.context.Context;
import org.tinygroup.context.impl.ContextImpl;
import org.tinygroup.logger.Logger;
import org.tinygroup.logger.LoggerFactory;
import org.tinygroup.mda.MDAFileCreator;
import org.tinygroup.mda.config.entity.EntityField;
import org.tinygroup.mda.config.entity.EntityGroup;
import org.tinygroup.mda.config.entity.EntityModel;
import org.tinygroup.mda.config.entity.Operation;
import org.tinygroup.mda.config.entity.View;
import org.tinygroup.mda.config.trans.MDACreatorConfig;
import org.tinygroup.mda.config.trans.MDACreators;
import org.tinygroup.mda.javabean.BeanDetial;
import org.tinygroup.mda.javabean.impl.MDATemplateHelper;
import org.tinygroup.mda.util.FileUtil;
import org.tinygroup.mda.util.InputModeUtil;
import org.tinygroup.mda.util.MDANameUtil;
import org.tinygroup.mda.util.MDAUtil;
import org.tinygroup.mda.util.ModelTransUtil;
import org.tinygroup.mda.util.OperationType;

public class EntityPageFileVelocityCreatorImpl extends MDATemplateHelper implements
		MDAFileCreator<EntityModel> {

	private final String DEFAULT_OP_ADD_TEMPLATE = "/pagetemplate/pageOpAdd.mdatemplate";
	private final String DEFAULT_OP_UPDATE_TEMPLATE = "/pagetemplate/pageOpUpdate.mdatemplate";
	private final String DEFAULT_VIEW_TEMPLATE = "/pagetemplate/pageView.mdatemplate";
	private final String DEFAULT_OP_VIEW_TEMPLATE = "/pagetemplate/pageOpView.mdatemplate";
	private final String OP_ADD_TEMPLATE = "add_template";
	private final String OP_UPDATE_TEMPLATE = "update_template";
	private final String VIEW_TEMPLATE = "page_view_template";
	private final String OP_VIEW_TEMPLATE = "view_template";
	
	// private final String DELETE_TEMPLATE = "/pageDelete.mdatemplate";
	// private final String QUERY_TEMPLATE = "/pageQuery.mdatemplate";
	private static Logger logger = LoggerFactory
			.getLogger(EntityPageFileVelocityCreatorImpl.class);

	public void createFile(EntityModel model, BeanDetial beanDetial,
			FileUtil fileUtil) {
		Context context = getContext();
		context.put("model", model);
		context.put("MDAUtil", new MDAUtil());
		context.put("MDANameUtil", new MDANameUtil());
		context.put("beanVarName", MDANameUtil.getMDAName( MDANameUtil.getBeanName(model.getName())));
		
		EntityField pField = null;
		for(EntityGroup group:model.getGroups()){
			for(EntityField field:group.getFields()){
				if(field.isPrimary())
					pField = field;
			}
		}
		if(pField == null){
			throw new RuntimeException("主键不存在");
		}
		context.put("primaryField", pField);
		createOpPageFiles(model, fileUtil, context);
		createViewPageFiles(model, fileUtil, context);
	}

	private Context getContext() {
		Context context = new ContextImpl();
		context.put("InputModeUtil", new InputModeUtil());
		return context;
	}

	private void createOpPageFiles(EntityModel model, FileUtil fileUtil,
			Context context) {
		for (Operation op : model.getOperations()) {
			createOpPageFile(model, fileUtil, context, op);
		}
	}
	
	private void createViewPageFiles(EntityModel model, FileUtil fileUtil,
			Context context) {
		for(View view:model.getViews()){
			createViewPageFile(model, fileUtil, context, view);
		}
		
	}
	private void createViewPageFile(EntityModel model, FileUtil fileUtil,
			Context context,View view) {
		context.put("view", view);
		String templatePath = templateMap.get(VIEW_TEMPLATE);
		if ("".equals(templatePath))
			return;
		try {
			String viewPageContent = ModelTransUtil.parseFile(context,
					templatePath);
			String opFilePackageName = MDANameUtil.getPagePackageName(model);
//			fileUtil.writePageFile(opFilePackageName,
//					MDANameUtil.getPageViewFileName(view.getName()), viewPageContent);
			fileUtil.writePageFile(opFilePackageName,
					MDANameUtil.getPageFileName(view.getName()), viewPageContent);
		} catch (Exception e) {
			logger.errorMessage("为模型视图生成Page时出错,模型:{0},视图:{1},模板路径:{2}", e,
					model.getName(), view.getName(),templatePath);
		}
	}
	private void createOpPageFile(EntityModel model, FileUtil fileUtil,
			Context context, Operation op) {
		String templatePath = getFlowTemplate(op.getType());
		
		if ("".equals(templatePath))
			return;
		try {
			context.put("operation", op);
			String opPageContent = ModelTransUtil.parseFile(context,
					templatePath);
			String opFilePackageName = MDANameUtil.getPagePackageName(model);
			fileUtil.writePageFile(opFilePackageName,
					MDANameUtil.getPageFileName(op.getName()), opPageContent);
		} catch (Exception e) {
			logger.errorMessage("为模型操作生成Page时出错,模型:{0},操作:{1},模板路径:{2}", e,
					model.getName(), op.getName(),templatePath);
		}
	}

	/**
	 * 获取不同操作对应的模板路径
	 * 
	 * @param opType
	 * @return
	 */
	private String getFlowTemplate(String opType) {
		if (opType.equals(OperationType.ADD.toString())) {
			return templateMap.get(OP_ADD_TEMPLATE);
		} else if (opType.equals(OperationType.DELETE.toString())) {
			return "";
		} else if (opType.equals(OperationType.QUERY.toString())) {
			return "";
		} else if (opType.equals(OperationType.UPDATE.toString())) {
			return templateMap.get(OP_UPDATE_TEMPLATE);
		} else if(opType.equals(OperationType.VIEW.toString())){
			return templateMap.get(OP_VIEW_TEMPLATE);
		} 
		return "";
	}

	protected void initTemplate() {
		checkTemplate(OP_ADD_TEMPLATE, DEFAULT_OP_ADD_TEMPLATE);
		checkTemplate(OP_UPDATE_TEMPLATE, DEFAULT_OP_UPDATE_TEMPLATE);
		checkTemplate(VIEW_TEMPLATE, DEFAULT_VIEW_TEMPLATE);
		checkTemplate(OP_VIEW_TEMPLATE, DEFAULT_OP_VIEW_TEMPLATE);
	}

	public void setCreatorConfig(MDACreatorConfig creator,MDACreators creators) {
		setTemplateMap(ModelTransUtil.getTemplateMap(creator));
	}


}
