/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.mda.transform.entity.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.tinygroup.database.config.table.Table;
import org.tinygroup.database.config.table.TableField;
import org.tinygroup.mda.config.entity.EntityField;
import org.tinygroup.mda.config.entity.EntityModel;
import org.tinygroup.mda.config.entity.Index;
import org.tinygroup.mda.javabean.BeanDetial;
import org.tinygroup.mda.javabean.BeanProperty;
import org.tinygroup.mda.transform.entity.EntityModelTransformer;
import org.tinygroup.mda.util.FileUtil;
import org.tinygroup.mda.util.MDANameUtil;
import org.tinygroup.mda.util.MDAUtil;
import org.tinygroup.metadata.config.stdfield.StandardField;

public class EntityModelTransformerImpl implements EntityModelTransformer {
	private EntityModel model;
	

	public EntityModelTransformerImpl(EntityModel model) {
		this.model = model;
	}

	public EntityModel getModle() {
		return model;
	}

	public Table tranformToTable(FileUtil fileUtil,
			Map<String, String> configMap) {
		Table table = new Table();
		table.setPackageName(model.getPackageName());
		String tablePre = "";
		if (configMap != null)
			tablePre = configMap.get(TABLE_PRE);
		table.setName(MDANameUtil.getTableName(model, tablePre));
		table.setId(MDANameUtil.getTableId(model));
		table.setTitle(model.getTitle());
		// fielduuid,tablefield
		Map<String, TableField> map = new HashMap<String, TableField>();
		dealFields(table, map);
		dealIndexs(table, map);

		fileUtil.writeTableFile(table);
		return table;
	}

	private void dealFields(Table table, Map<String, TableField> map) {
		List<EntityField> fields = MDAUtil.getFields(model);
		// 生成字段
		for (EntityField field : fields) {
			TableField tf = new TableField();
			tf.setStandardFieldId(field.getStandardFieldId());
			tf.setId(field.getUuid());
			tf.setPrimary(field.isPrimary());
			if(tf.getPrimary()){//如果是主键，则设置为自增长
				tf.setAutoIncrease(true);
			}
			tf.setUnique(field.isUnique());
			// TODO:模型字段上没有not-null
			table.getFieldList().add(tf);
			map.put(field.getUuid(), tf);
		}
	}

	private void dealIndexs(Table table, Map<String, TableField> map) {
		// 生成索引
		for (Index index : model.getIndexs()) {
			org.tinygroup.database.config.table.Index tableIndex = new org.tinygroup.database.config.table.Index();
			tableIndex.setName(index.getName());
			tableIndex.setUnique(index.isUnique());
			// TODO:模型Index上没有是否反转索引
			for (String modelFieldUUID : index.getFields()) {
				TableField tf = map.get(modelFieldUUID);
				if (tf != null) {
					tableIndex.getFields().add(tf.getId());
				}
			}
			table.getIndexList().add(tableIndex);
		}
	}

	public BeanDetial tranformToBeanDetial() {
		BeanDetial beanDetial = new BeanDetial();
		beanDetial.setName(MDANameUtil.getBeanName(model.getName()));
		beanDetial.setPackageName(model.getPackageName());
		for (EntityField field : MDAUtil.getFields(model)) {
			String stduuid = field.getStandardFieldId();
			StandardField propertyField = MDAUtil.getField(stduuid);
			BeanProperty beanProperty = new BeanProperty();
			beanProperty.setJavaType(MDAUtil.getFieldType(stduuid, "java"));
			beanProperty
					.setName(MDANameUtil.getMDAName(propertyField.getName()));
			beanProperty.setPrimary(field.isPrimary());
			beanDetial.addProperty(beanProperty);
		}

		return beanDetial;
	}

}
