/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.mda.transform.view.impl;

import java.util.HashMap;
import java.util.Map;

import org.tinygroup.database.config.view.Having;
import org.tinygroup.database.config.view.OrderByField;
import org.tinygroup.database.config.view.View;
import org.tinygroup.database.config.view.ViewCondition;
import org.tinygroup.database.config.view.ViewField;
import org.tinygroup.database.config.view.ViewFieldRef;
import org.tinygroup.database.config.view.ViewHaving;
import org.tinygroup.database.config.view.ViewTable;
import org.tinygroup.mda.ModelManagerFactory;
import org.tinygroup.mda.config.ConditionField;
import org.tinygroup.mda.config.ConditionValue;
import org.tinygroup.mda.config.OrderField;
import org.tinygroup.mda.config.entity.EntityField;
import org.tinygroup.mda.config.entity.EntityModel;
import org.tinygroup.mda.config.view.ModelRelation;
import org.tinygroup.mda.config.view.RelateModel;
import org.tinygroup.mda.config.view.ViewModel;
import org.tinygroup.mda.config.view.ViewModelField;
import org.tinygroup.mda.javabean.BeanDetial;
import org.tinygroup.mda.javabean.BeanProperty;
import org.tinygroup.mda.transform.view.ViewModelTransformer;
import org.tinygroup.mda.util.FileUtil;
import org.tinygroup.mda.util.MDANameUtil;
import org.tinygroup.mda.util.MDAUtil;
import org.tinygroup.metadata.config.stdfield.StandardField;

public class ViewModelTransformerImpl implements ViewModelTransformer {
	private ViewModel model;
	private String VIEW_PRE = "view_pre";

	public ViewModelTransformerImpl(ViewModel model) {
		this.model = model;
	}

	public ViewModel getModle() {
		return model;
	}

	public View tranformToView(FileUtil fileUtil,Map<String, String> configMap) {
		View tableView = new View();
		String viewPre = "";
		if(configMap!=null)
			viewPre = configMap.get(VIEW_PRE);
		tableView.setName(MDANameUtil.getViewName(model,viewPre));
		Map<String, ViewTable> tableMap = new HashMap<String, ViewTable>();
		Map<String, ViewField> map = new HashMap<String, ViewField>();
		dealRelateModels(tableView, tableMap);
		dealModelRelations(tableView, tableMap);
		dealFields(tableView, map);
		dealOrderByField(tableView, map, tableMap);
		// dealGroupByField(tableView, map);
		dealConditionFields(tableView, map, tableMap);
		model.getConditionFields();
		fileUtil.writeViewFile(tableView);
		return tableView;
	}

	private void dealConditionFields(View tableView,
			Map<String, ViewField> map, Map<String, ViewTable> tableMap) {
		for (ConditionField field : model.getConditionFields()) {
			if (field.getAggregateFunction() == null
					|| field.getAggregateFunction().equals("")) {
				// 无聚合函数配置则声称where条件
				conditionFieldToCondition(tableView, field, map, tableMap);
			} else { // 有则生成having语句
				conditionFieldToConditionHaving(tableView, field, map, tableMap);
			}

		}
	}

	private void conditionFieldToConditionHaving(View tableView,
			ConditionField field, Map<String, ViewField> map,
			Map<String, ViewTable> tableMap) {
		ViewHaving having = new ViewHaving();
		// 处理keyField;
		Having keyHaving = new Having();
		keyHaving.setAggregateFunction(field.getAggregateFunction());
		if (field.getModeluuid() == null) {
			ViewFieldRef vf = new ViewFieldRef();
			vf.setViewFieldId(map.get(field.getFielduuid()).getId());
			keyHaving.setField(vf);
		} else {
			ViewFieldRef vf = new ViewFieldRef();
			vf.setViewTableId(tableMap.get(field.getModeluuid()).getId());
			vf.setTableFieldId(field.getFielduuid());
			keyHaving.setField(vf);
		}
		// TODO:operator这里要修改
		having.setOperator(field.getCompareMode());
		having.setKeyHaving(keyHaving);

		ConditionValue value = field.getValue();
		if (value.getConstValue() == null) {
			Having valueHaving = new Having();
			valueHaving.setAggregateFunction(value.getAggregateFunction());
			ViewFieldRef vf = new ViewFieldRef();
			vf.setViewTableId(tableMap.get(value.getModeluuid()).getId());
			vf.setTableFieldId(value.getFielduuid());
			valueHaving.setField(vf);
			having.setValueHaving(valueHaving);
		} else {
			having.setValue(value.getConstValue());
		}
		tableView.getHavingList().add(having);
	}

	private void conditionFieldToCondition(View tableView,
			ConditionField field, Map<String, ViewField> map,
			Map<String, ViewTable> tableMap) {
		ViewCondition condition = new ViewCondition();
		// 处理keyField;
		if (field.getModeluuid() == null) {
			ViewFieldRef vf = new ViewFieldRef();
			vf.setViewFieldId(map.get(field.getFielduuid()).getId());
			condition.setKeyField(vf);
		} else {
			ViewFieldRef vf = new ViewFieldRef();
			vf.setViewTableId(tableMap.get(field.getModeluuid()).getId());
			vf.setTableFieldId(field.getFielduuid());
			condition.setKeyField(vf);
		}
		// TODO:operator这里要修改
		condition.setOperator(field.getCompareMode());

		// 条件值一定是固定值或者是一个字段
		// 处理valueField;
		ConditionValue value = field.getValue();
		if (value.getConstValue() != null) {
			condition.setValue(value.getConstValue());
		} else {
			ViewFieldRef vf = new ViewFieldRef();
			vf.setViewTableId(tableMap.get(value.getModeluuid()).getId());
			vf.setTableFieldId(value.getFielduuid());
			condition.setValueField(vf);
		}
		tableView.getConditionList().add(condition);
	}

	// private void dealGroupByField(View tableView, Map<String, ViewField> map)
	// {
	// // 设置分组字段
	// for (GroupField field : model.getGroupFields()) {
	// GroupByField groupByField = new GroupByField();
	// if (field.getModeluuid() == null) {// 如果模型id为空，则代表为当前模型字段
	// groupByField.setField(map.get(field.getFielduuid()));
	// } else {
	// groupByField.setField(getViewFieldRef(field.getModeluuid(),
	// field.getFielduuid()));
	// }
	// tableView.getGroupByFieldList().add(groupByField);
	// }
	// }

	private void dealOrderByField(View tableView, Map<String, ViewField> map,
			Map<String, ViewTable> tableMap) {
		// 设置排序字段
		for (OrderField orderField : model.getOrderFields()) {
			OrderByField orderByField = new OrderByField();
			if (orderField.getModeluuid() == null) {// 如果模型id为空，则代表为当前模型字段
				ViewFieldRef vf = new ViewFieldRef();
				vf.setViewFieldId(map.get(orderField.getFielduuid()).getId());
				orderByField.setField(vf);
			} else {
				orderByField.setField(getViewFieldRef(
						orderField.getModeluuid(), orderField, tableMap));
			}
			orderByField.setDirection(orderField.getOrderMode());
			tableView.getOrderByFieldList().add(orderByField);
		}
	}

	/**
	 * 根据 关联模型id与关联模型字段id生成一个ViewField 该场景主要应用在
	 * where/orderby/groupby/having等子句中出现了某个不存在于视图模型字段列表中的字段
	 * 
	 * @param relateModeluuid
	 * @param modelFielduuid
	 * @return
	 */
	private ViewFieldRef getViewFieldRef(String relateModeluuid,
			OrderField orderField, Map<String, ViewTable> tableMap) {
		String modelFielduuid = orderField.getFielduuid();
		ViewFieldRef vf = new ViewFieldRef();
		RelateModel model = getRelateModel(relateModeluuid);
		// EntityModel entity = ModelManagerFactory.getManager().getEntityModel(
		// model.getEntityuuid());
		// EntityField entityField = MDAUtil.getField(entity, modelFielduuid);
		vf.setTableFieldId(modelFielduuid);
		// (getTableName(model));
		vf.setViewTableId((tableMap.get(model.getUuid()).getId()));
		return vf;
	}

	private void dealFields(View tableView, Map<String, ViewField> map) {
		// 生成视图字段
		for (ViewModelField field : MDAUtil.getViewModelFields(model)) {
			RelateModel relatemodel = MDAUtil.getRelateModel(model,
					field.getRelateModelUuid());
			EntityModel entity = ModelManagerFactory.getManager()
					.getEntityModel(relatemodel.getEntityuuid());
			EntityField entityField = MDAUtil.getField(entity,
					field.getEntityFieldduuid());
			ViewField vf = new ViewField();
			vf.setAlias(field.getAlias());
			vf.setTableFieldId(entityField.getUuid());
			vf.setId(field.getUuid());
			vf.setViewTable(relatemodel.getUuid());
			tableView.getFieldList().add(vf);
			map.put(field.getUuid(), vf);
		}
	}

	private void dealModelRelations(View tableView,
			Map<String, ViewTable> tableMap) {
		// 模型的模型关系列表，生成为view的条件列表
		for (ModelRelation relation : model.getRelations()) {
			dealModelRelation(tableView, relation, tableMap);
		}
	}

	private void dealModelRelation(View tableView, ModelRelation relation,
			Map<String, ViewTable> tableMap) {
		RelateModel rm = getRelateModel(relation.getRelateModel());
		RelateModel rm2 = getRelateModel(relation.getRelateModel2());
		EntityModel entity = ModelManagerFactory.getManager().getEntityModel(
				rm.getEntityuuid());
		EntityModel entity2 = ModelManagerFactory.getManager().getEntityModel(
				rm2.getEntityuuid());
		// 拼装 entity.xxx=
		ViewCondition condition = new ViewCondition();
		condition.setOperator("=");

		ViewFieldRef keyField = new ViewFieldRef();
		keyField.setViewTableId(tableMap.get(rm.getUuid()).getId());
		EntityField foreignField = null;
		for (EntityField field : MDAUtil.getFields(entity)) {
			if (rm2.getEntityuuid().equals(field.getEntityuuid())) { // 如果是那个外键
				foreignField = field;
				break;
			}
		}
		// foreignField不可能为空
		keyField.setTableFieldId(foreignField.getUuid());
		condition.setKeyField(keyField);
		// 拼装entity2.id
		ViewFieldRef valueField = new ViewFieldRef();
		valueField.setViewTableId(tableMap.get(rm2.getUuid()).getId());
		valueField.setTableFieldId(MDAUtil.getPrimarykey(entity2).getUuid());
		// 生成条件
		// entity.xxx=entity2.id
		condition.setValueField(valueField);
		tableView.getConditionList().add(condition);
	}

	private void dealRelateModels(View tableView,
			Map<String, ViewTable> tableMap) {
		// 模型的关联模型列表，生成为view的关联表
		for (RelateModel relateModel : model.getRelateModels()) {
			String entityuuid = relateModel.getEntityuuid();
			String alias = relateModel.getEntityalias();
			EntityModel entity = ModelManagerFactory.getManager()
					.getEntityModel(entityuuid);
			ViewTable vt = createViewTable(alias, relateModel, entity);
			tableMap.put(relateModel.getUuid(), vt);
			tableView.getTableList().add(vt);
		}
	}

	// private String getTableName(RelateModel model) {
	// if (model.getEntityalias() != null
	// || !model.getEntityalias().equals(""))
	// return model.getEntityalias();
	// EntityModel entity = ModelManagerFactory.getManager().getEntityModel(
	// model.getEntityuuid());
	// return entity.getName();
	//
	// }

	private ViewTable createViewTable(String alias, RelateModel relateModel,
			EntityModel entity) {
		ViewTable vt = new ViewTable();
		vt.setId(relateModel.getUuid());
		vt.setTableAlias(alias);
		vt.setTableId(MDANameUtil.getTableId(entity));
		return vt;
	}

	private RelateModel getRelateModel(String uuid) {
		for (RelateModel rm : model.getRelateModels()) {
			if (rm.getUuid().equals(uuid)) {
				return rm;
			}
		}
		return null;
	}

	public BeanDetial tranformToBeanDetial() {
		BeanDetial beanDetial = new BeanDetial();
		beanDetial.setName(MDANameUtil.getBeanName(model.getName()));
		beanDetial.setPackageName(model.getPackageName());
		for (ViewModelField field : MDAUtil.getFields(model)) {
			String entityFieldUUID = field.getEntityFieldduuid();
			String relateModelUUID = field.getRelateModelUuid();
			RelateModel relateModel = getRelateModel(relateModelUUID);
			EntityModel entityModel = ModelManagerFactory.getManager()
					.getEntityModel(relateModel.getEntityuuid());
			EntityField entityField = MDAUtil.getField(entityModel,
					entityFieldUUID);
			// beanDef.addProperty(entityField.getStandardFieldId());

			String stduuid = entityField.getStandardFieldId();
			StandardField propertyField = MDAUtil.getField(stduuid);
			BeanProperty beanProperty = new BeanProperty();
			beanProperty.setJavaType(MDAUtil.getFieldType(stduuid, "java"));
			beanProperty.setName(MDANameUtil.getMDAName(
					propertyField.getName()));
			beanDetial.addProperty(beanProperty);
		}
		return beanDetial;
	}
}
