/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.mda.util;

import java.io.File;
import java.io.IOException;

import org.tinygroup.commons.file.FileDealUtil;
import org.tinygroup.database.config.table.Table;
import org.tinygroup.database.config.table.Tables;
import org.tinygroup.database.config.view.View;
import org.tinygroup.database.config.view.Views;
import org.tinygroup.database.util.DataBaseUtil;
import org.tinygroup.flow.FlowExecutor;
import org.tinygroup.flow.config.Flow;
import org.tinygroup.logger.LogLevel;
import org.tinygroup.logger.Logger;
import org.tinygroup.logger.LoggerFactory;
import org.tinygroup.service.Service;
import org.tinygroup.service.config.ServiceComponents;
import org.tinygroup.xstream.XStreamFactory;

import com.thoughtworks.xstream.XStream;

/**
 * 
 * @author chenjiao
 * 
 */
public class FileUtil {
	private static Logger logger = LoggerFactory.getLogger(FileUtil.class);
	private static String resourcePath = "";
	private static String srcPath = "";
	public FileUtil(String src, String resource) {
		srcPath = src;
		resourcePath = resource;
	}

	private String getResource() {
		return resourcePath;
	}

	private String getSrc() {
		return srcPath;
	}

	public void writeFile(File file, String content) {
		try {
			FileDealUtil.write(file, content);
		} catch (IOException e) {
			logger.errorMessage("写文件时发生错误，文件名:{0}", e, file.getName());
		}
	}

	public void writePageFile(String packageName, String fileName,
			String content) {
		logger.logMessage(LogLevel.INFO, "开始创建page文件,名:{0}", fileName);
		String fileDir = getResourceDir(packageName);
		writeFile(fileDir, fileName, content);
		logger.logMessage(LogLevel.INFO, "创建page文件完成,名:{0}", fileName);
	}

	public void writeTableFile(Table table) {
		logger.logMessage(LogLevel.INFO, "开始创建table文件,包:{0},名:{1}",
				table.getPackageName(), table.getName());
		XStream stream = XStreamFactory
				.getXStream(DataBaseUtil.DATABASE_XSTREAM);
		String fileDir = getResourceDir(table.getPackageName());
		;
		String fileName = String.format("%s.table.xml", table.getName());
		Tables tables = new Tables();
		tables.getTableList().add(table);
		tables.setPackageName(table.getPackageName());
		String tablesXml = stream.toXML(tables);
		writeFile(fileDir, fileName, tablesXml);
		logger.logMessage(LogLevel.INFO, "创建table文件完成,包:{0},名:{1}",
				table.getPackageName(), table.getName());
	}

	public void writeViewFile(View view) {
		logger.logMessage(LogLevel.INFO, "开始创建view文件,名:{0}", view.getName());
		XStream stream = XStreamFactory
				.getXStream(DataBaseUtil.DATABASE_XSTREAM);
		String topDir = getResource();
		String fileName = String.format("%s.view.xml", view.getName());
		Views views = new Views();
		views.getViewTableList().add(view);
		String viewsXml = stream.toXML(views);
		writeFile(topDir, fileName, viewsXml);
		logger.logMessage(LogLevel.INFO, "创建view文件完成,名:{0}", view.getName());
	}

	public void writeJavaFile(String packageName, String className,
			String content) {
		logger.logMessage(LogLevel.INFO, "开始创建java类,包:{0},名:{1}", packageName,
				className);
		String fileDir = getSrcDir(packageName);
		String fileName = String.format("%s.java", className);
		writeFile(fileDir, fileName, content);
		logger.logMessage(LogLevel.INFO, "创建java类完成,包:{0},名:{1}", packageName,
				className);

	}

	public void writeServiceFile(String packageName, String className,
			ServiceComponents service) {
		logger.logMessage(LogLevel.INFO, "开始创建service配置文件,包:{0},类:{1}",
				packageName, className);
		String fileDir = getResourceDir(packageName);
		String fileName = String.format("%s.service.xml", className);
		XStream stream = XStreamFactory
				.getXStream(Service.SERVICE_XSTREAM_PACKAGENAME);
		String content = stream.toXML(service);
		writeFile(fileDir, fileName, content);
		logger.logMessage(LogLevel.INFO, "创建service配置文件完成,包:{0},类:{1}",
				packageName, className);
	}
	
	public void writeServiceFile(String packageName, String className,
			String serviceContent) {
		logger.logMessage(LogLevel.INFO, "开始创建service配置文件,包:{0},类:{1}",
				packageName, className);
		String fileDir = getResourceDir(packageName);
		String fileName = String.format("%s.service.xml", className);
		writeFile(fileDir, fileName, serviceContent);
		logger.logMessage(LogLevel.INFO, "创建service配置文件完成,包:{0},类:{1}",
				packageName, className);
	}
	

	public void writeFlowFile(String packageName, Flow flow) {
		logger.logMessage(LogLevel.INFO, "开始创建flow配置文件,包:{0},名:{1}",
				packageName, flow.getName());
		String fileDir = getResourceDir(packageName);
		String fileName = String.format("%s.pageflow.xml", flow.getId());
		XStream stream = XStreamFactory
				.getXStream(FlowExecutor.FLOW_XSTREAM_PACKAGENAME);
		String content = stream.toXML(flow);
		writeFile(fileDir, fileName, content);
		logger.logMessage(LogLevel.INFO, "创建flow配置文件完成,包:{0},名:{1}",
				packageName, flow.getName());

	}
	
	public void writePageFlowFile(String packageName, String flowId,String flowContent) {
		logger.logMessage(LogLevel.INFO, "开始创建flow配置文件,包:{0},ID:{1}",
				packageName, flowId);
		String fileDir = getResourceDir(packageName);
		String fileName = String.format("%s.pageflow.xml", flowId);
		writeFile(fileDir, fileName, flowContent);
		logger.logMessage(LogLevel.INFO, "创建flow配置文件完成,包:{0},ID:{1}",
				packageName, flowId);

	}

	private String getSrcDir(String packageName) {
		String topDir = getSrc();
		return String.format("%s/%s", topDir, getFilePath(packageName));
	}

	private String getResourceDir(String packageName) {
		String topDir = getResource();
		return String.format("%s/%s", topDir, getFilePath(packageName));
	}

	private void writeFile(String fileDir, String fileName, String content) {
		logger.logMessage(LogLevel.INFO, "开始创建文件，包:{0},文件名:{1}", fileDir,
				fileName);
		File dirFile = new File(fileDir);
		if (!dirFile.exists()) {
			dirFile.mkdirs();
		}
		String filePath = String.format("%s/%s", fileDir, fileName);
		File file = new File(filePath);

		if (!file.exists()) {
			try {
				file.createNewFile();
			} catch (IOException e) {
				logger.errorMessage( "创建文件时出错，文件名:{0}", e,filePath);
				
			}
		}
		writeFile(file, content);
		logger.logMessage(LogLevel.INFO, "创建文件完成，包:{0},文件名:{1}", fileDir,
				fileName);
	}

	public String getFilePath(String packageName) {
		return packageName.replace(".", "/");
	}

	public void writeSpringFile(String packageName, String beanName,
			String springContent) {
//		String fileDir = getResourceDir(packageName);
//		//String springDaoId = MDANameUtil.getSpringDaoId(beanDetial.getName());
//		String beanClassName = String.format("%s.%s",packageName,className);
//		SpringDaoFileCreator fileCreator=new SpringDaoFileCreator();
//		fileCreator.writeSpringFile(getResource(),fileDir,springBeanId,beanClassName,propertys);
		logger.logMessage(LogLevel.INFO, "开始创建springbeans配置文件,文件包路径:{0}业务对象名:{1}",
				packageName, beanName);
		String fileDir = getResourceDir(packageName);
		String fileName = String.format("%s.springbeans.xml", beanName);
		writeFile(fileDir, fileName, springContent);
		logger.logMessage(LogLevel.INFO, "创建springbeans配置文件完成,文件包路径:{0}业务对象名:{1}",
				packageName, beanName);
		
		
	}

	public void writeCommonFile(String packageName, String fileName,
			String content) {
		logger.logMessage(LogLevel.INFO, "开始创建文件,包:{0},名:{1}", packageName,
				fileName);
		String fileDir = getSrcDir(packageName);
		writeFile(fileDir, fileName, content);
		logger.logMessage(LogLevel.INFO, "创建文件完成,包:{0},名:{1}", packageName,
				fileName);

	}

}
