/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.mda.util;

import java.util.ArrayList;
import java.util.List;

import org.tinygroup.mda.config.entity.EntityField;
import org.tinygroup.mda.config.entity.EntityGroup;
import org.tinygroup.mda.config.entity.EntityModel;
import org.tinygroup.mda.config.view.RelateModel;
import org.tinygroup.mda.config.view.ViewModel;
import org.tinygroup.mda.config.view.ViewModelField;
import org.tinygroup.mda.config.view.ViewModelGroup;
import org.tinygroup.metadata.config.stdfield.StandardField;
import org.tinygroup.metadata.stdfield.StandardFieldProcessor;
import org.tinygroup.metadata.util.MetadataUtil;
import org.tinygroup.springutil.SpringUtil;

public class MDAUtil {
	/**
	 * 返回视图模型的所有字段
	 * 
	 * @param model
	 * @return
	 */
	public static List<ViewModelField> getViewModelFields(ViewModel model) {
		List<ViewModelField> list = new ArrayList<ViewModelField>();
		for (ViewModelGroup group : model.getViewGroups()) {
			list.addAll(group.getFields());
		}
		return list;
	}

	/**
	 * 返回实体模型的所有字段
	 * 
	 * @param model
	 * @return
	 */
	public static List<EntityField> getFields(EntityModel model) {
		List<EntityField> list = new ArrayList<EntityField>();
		for (EntityGroup group : model.getGroups()) {
			list.addAll(group.getFields());
		}
		return list;
	}

	/**
	 * 返回实体模型的所有字段
	 * 
	 * @param model
	 * @return
	 */
	public static List<ViewModelField> getFields(ViewModel model) {
		List<ViewModelField> list = new ArrayList<ViewModelField>();
		for (ViewModelGroup group : model.getViewGroups()) {
			list.addAll(group.getFields());
		}
		return list;
	}

	/**
	 * 返回实体模型的主键
	 * 
	 * @param model
	 * @return
	 */
	public static EntityField getPrimarykey(EntityModel model) {
		for (EntityGroup group : model.getGroups()) {
			for (EntityField field : group.getFields()) {
				if (field.isPrimary())
					return field;
			}
		}
		return null;
	}

	/**
	 * 根据模型字段UUID从实体模型中获取模型字段
	 * 
	 * @param model
	 * @param uuid
	 * @return
	 */
	public static EntityField getField(EntityModel model, String uuid) {
		for (EntityGroup group : model.getGroups()) {
			for (EntityField field : group.getFields()) {
				if (field.getUuid().equals(uuid))
					return field;
			}
		}
		return null;
	}

	public static ViewModelField getField(ViewModel model, String uuid) {
		for (ViewModelGroup group : model.getViewGroups()) {
			for (ViewModelField field : group.getFields()) {
				if (field.getUuid().equals(uuid)) {
					return field;
				}
			}
		}
		return null;
	}

	public static RelateModel getRelateModel(ViewModel model, String uuid) {
		if (uuid == null || uuid.equals(""))
			return null;

		for (RelateModel relateModel : model.getRelateModels()) {
			if (uuid.equals(relateModel.getUuid())) {
				return relateModel;
			}
		}
		return null;
	}

	public static StandardField getField(String stdFieldId) {
		StandardFieldProcessor standardFieldProcessor = SpringUtil.getBean(MetadataUtil.STDFIELDPROCESSOR_BEAN);
		return standardFieldProcessor.getStandardField(stdFieldId);
	}

	public static String getFieldType(String stdFieldId, String type) {
		StandardFieldProcessor standardFieldProcessor = SpringUtil.getBean(MetadataUtil.STDFIELDPROCESSOR_BEAN);
		return standardFieldProcessor.getType(stdFieldId, type);
	}

	
//	private static String getFields(QueryObject obj) {
//		if (obj.getFields().size() == 0) {
//			return " * ";
//		}
////		String entityName = obj.getEntityName();
//		String fieldsStr = "";
//		for (String field : obj.getFields()) {
//			if (fieldsStr.equals("")) {
//				fieldsStr = field;
////				fieldsStr = entityName + "." + field;
//			} else {
//				fieldsStr += "," + field;
////				fieldsStr += "," + entityName + "." + field;
//			}
//		}
//		return fieldsStr;
//	}
}
