/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.mda.util;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.tinygroup.context.Context;
import org.tinygroup.logger.Logger;
import org.tinygroup.logger.LoggerFactory;
import org.tinygroup.mda.ModelManagerFactory;
import org.tinygroup.mda.config.entity.EntityModel;
import org.tinygroup.mda.config.trans.MDAConfig;
import org.tinygroup.mda.config.trans.MDACreatorConfig;
import org.tinygroup.mda.config.trans.MDACreators;
import org.tinygroup.mda.config.trans.MDATemplate;
import org.tinygroup.mda.config.view.ViewModel;
import org.tinygroup.parser.Document;
import org.tinygroup.tinytestutil.AbstractTestUtil;
import org.tinygroup.velocity.impl.VelocityHelperImpl;
import org.tinygroup.xmlparser.node.XmlNode;
import org.tinygroup.xmlparser.parser.XmlParser;
import org.tinygroup.xmlparser.parser.XmlStringParser;
import org.tinygroup.xstream.XStreamFactory;

import com.thoughtworks.xstream.XStream;

public class ModelTransUtil{
	static boolean inited = false;
//	private static FullContextFileRepository repository;
	private static VelocityHelperImpl helper ;//= new VelocityHelperImpl();
	private static final String CONFIG_FILE = "mdaconfig.xml";
	private static Logger logger = LoggerFactory.getLogger(ModelTransUtil.class);
	private static List<MDACreators> creatorsList = new ArrayList<MDACreators>();
	
	public static List<MDACreators> getAllCreators(){
		return creatorsList;
	}
	
	public static String parseFile(Context context, String templatePath)
			throws FileNotFoundException, Exception {
		StringWriter writer = new StringWriter();
		helper.processTempleate(context, writer, templatePath);
		return writer.toString();
	}

	public static void init(List<String> classPaths, String resourcePath) {
		if (!inited) {
			inited = true;
			initFileResolver(classPaths);
			try {
				initConfig(resourcePath);
			} catch (IOException e) {
				creatorsList.clear();
				logger.errorMessage("读取mdaconfig.xml出错", e);
			}
		}
	}

	private static void initConfig(String resourcePath) throws IOException {
		XStream xstream = XStreamFactory.getXStream("mda");
		
		String configFile = resourcePath + File.separator + CONFIG_FILE;
		File file = new File(configFile);
		if(!file.exists()){
			return ;
		}
		MDAConfig mdaConfig = (MDAConfig)xstream.fromXML(file);
		parseMDAConfig(mdaConfig);
	}
	
	private static void parseMDAConfig(MDAConfig config){
		for(MDACreators creators:config.getCreators()){
			creatorsList.add(creators);
		}
	}

	private static void initFileResolver(List<String> classPaths) {
		List<String> realPaths = new ArrayList<String>();
		realPaths.add(ModelTransUtil.class.getResource("").getPath());
		realPaths.addAll(classPaths);
		String config="<full-context-file-finder>"
				+ "<file ext-name=\"mdatemplate\" content-type=\"application/test\" />"
				+ "</full-context-file-finder>";
		XmlParser<String> parse = new XmlStringParser();
		Document<XmlNode> document = parse.parse(config);
		AbstractTestUtil.init("/compile/MDAComplie.Application.preloadbeans.xml", true,document.getRoot(),realPaths);
		helper = AbstractTestUtil.getVelocityHelper();
//		Factory factory = BeanFactory.getFactory();
//		XStream xStream = XStreamFactory.getXStream();
//		factory.addBeans((Beans) xStream.fromXML(AbstractTestUtil.class
//				.getResourceAsStream("/compile/MDAComplie.Application.preloadbeans.xml")));
//		factory.init();
//		FileResolver fileResolver = factory.getBean("fileResolver");
//
//		repository = new FullContextFileRepositoryImpl();
//		helper.setFullContextFileRepository(repository);
//		FullContextFileFinder finder = new FullContextFileFinder();
//		XmlParser<String> parse = new XmlStringParser();
//		Document<XmlNode> document = parse
//				.parse("<full-context-file-finder>"
//						+ "<file ext-name=\"mdatemplate\" content-type=\"application/test\" />"
//						+ "</full-context-file-finder>");
//		finder.setConfiguration(document.getRoot());
//		finder.setFullContextFileRepository(repository);
//		fileResolver.addFileProcessor(finder);
//		repository
//				.addSearchPath(ModelTransUtil.class.getResource("").getPath());
//		if (classPaths != null) {
//			for (String path : classPaths) {
//				fileResolver.addManualClassPath(path);
//				repository.addSearchPath(path);
//			}
//		}
//		fileResolver.resolve();
	}

	public static EntityModel getEntity(String uuid) {
		return ModelManagerFactory.getManager().getEntityModel(uuid);
	}

	public static ViewModel getViewModel(String uuid) {
		return ModelManagerFactory.getManager().getViewModel(uuid);
	}
	
	public static Map<String,String> getTemplateMap(MDACreatorConfig creator){
		Map<String, String> templateMap = new HashMap<String, String>();
		if(creator==null)
			return templateMap;
		for(MDATemplate template:creator.getTemplates()){
			templateMap.put(template.getKey(), template.getValue());
		}
		return templateMap;
	}
	public static String getTemplate1(MDACreatorConfig creator,String key,String defaultValue){
		if(creator==null||creator.getTemplates()==null)
			return defaultValue;
		String templatePath = "";
		for(MDATemplate template:creator.getTemplates()){
			if(key.equals(template.getKey())){
				templatePath =  template.getValue();
				break;
			}
		}
		if(templatePath==null||"".equals(templatePath)){
			templatePath = defaultValue;
		}
		return templatePath;
	}

	
}
